@@ -180,14 +180,18 @@ class SOCol(object):
         self._default = default
         self.customSQLType = sqlType
 
+        # deal with foreign keys
         self.foreignKey = foreignKey
         if self.foreignKey:
-            #assert self.name.upper().endswith('ID'), "All foreign key columns must end with 'ID' (%s)" % repr(self.name)
-            if not self.name.upper().endswith('ID'):
-                self.foreignName = self.name
-                self.name = self.name + "ID"
+            if origName is not None:
+                idname = soClass.sqlmeta.style.instanceAttrToIDAttr(origName)
             else:
-                self.foreignName = self.name[:-2]
+                idname = soClass.sqlmeta.style.instanceAttrToIDAttr(name)
+            if self.name != idname:
+                self.foreignName = self.name
+                self.name = idname
+            else:
+                self.foreignName = soClass.sqlmeta.style.instanceIDAttrToAttr(self.name)
         else:
             self.foreignName = None
 
@@ -265,8 +269,7 @@ class SOCol(object):
     default = property(_get_default, None, None)
 
     def _get_joinName(self):
-        assert self.name[-2:] == 'ID'
-        return self.name[:-2]
+        return self.soClass.sqlmeta.style.instanceIDAttrToAttr(self.name)
     joinName = property(_get_joinName, None, None)
 
     def __repr__(self):
@@ -698,30 +701,21 @@ class FloatCol(Col):
 
 
 class SOKeyCol(SOCol):
+    key_type = {int: "INT", str: "TEXT"}
 
     # 3-03 @@: this should have a simplified constructor
     # Should provide foreign key information for other DBs.
 
-    def _mysqlType(self):
-        return 'INT'
-
-    def _sqliteType(self):
-        return 'INT'
-
-    def _postgresType(self):
-        return 'INT'
+    def _sqlType(self):
+        return self.key_type[self.soClass.sqlmeta.idType]
 
     def _sybaseType(self):
-        return 'NUMERIC(18,0) NULL'
+        key_type = {int: "NUMERIC(18,0) NULL", str: "TEXT"}
+        return key_type[self.soClass.sqlmeta.idType]
 
     def _mssqlType(self):
-        return 'INT NULL'
-
-    def _firebirdType(self):
-        return 'INT'
-
-    def _maxdbType(self):
-        return 'INT'
+        key_type = {int: "INT NULL", str: "TEXT"}
+        return key_type[self.soClass.sqlmeta.idType]
 
 class KeyCol(Col):
 
@@ -736,8 +730,7 @@ class SOForeignKey(SOKeyCol):
             kw['name'] = style.instanceAttrToIDAttr(style.pythonClassToAttr(foreignKey))
         else:
             kw['origName'] = kw['name']
-            if not kw['name'].upper().endswith('ID'):
-                kw['name'] = style.instanceAttrToIDAttr(kw['name'])
+            kw['name'] = style.instanceAttrToIDAttr(kw['name'])
         super(SOForeignKey, self).__init__(**kw)
 
     def sqliteCreateSQL(self):
@@ -767,6 +760,10 @@ class SOForeignKey(SOKeyCol):
 
     def postgresCreateSQL(self):
         sql = SOKeyCol.postgresCreateSQL(self)
+        return sql
+
+    def postgresCreateReferenceConstraint(self):
+        sTName = self.soClass.sqlmeta.table
         other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
         tName = other.sqlmeta.table
         idName = other.sqlmeta.idName
@@ -779,19 +776,19 @@ class SOForeignKey(SOKeyCol):
                 action = 'ON DELETE RESTRICT'
         else:
             action = ''
-        constraint = ('CONSTRAINT %(colName)s_exists '
+        constraint = ('ALTER TABLE %(sTName)s ADD CONSTRAINT %(colName)s_exists '
                       'FOREIGN KEY (%(colName)s) '
                       'REFERENCES %(tName)s (%(idName)s) '
                       '%(action)s' %
                       {'tName': tName,
                        'colName': self.dbName,
                        'idName': idName,
-                       'action': action})
-        sql = ', '.join([sql, constraint])
-        return sql
+                       'action': action,
+                       'sTName': sTName})
+        return constraint
 
-    def postgresCreateColumnSQL(self):
-        sql = SOKeyCol.postgresCreateSQL(self)
+    def mysqlCreateReferenceConstraint(self):
+        sTName = self.soClass.sqlmeta.table
         other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
         tName = other.sqlmeta.table
         idName = other.sqlmeta.idName
@@ -804,15 +801,19 @@ class SOForeignKey(SOKeyCol):
                 action = 'ON DELETE RESTRICT'
         else:
             action = ''
-        constraint = ('CONSTRAINT %(colName)s_exists '
+        constraint = ('ALTER TABLE %(sTName)s ADD CONSTRAINT %(sTName)s_%(colName)s_exists '
+                      'FOREIGN KEY (%(colName)s) '
                       'REFERENCES %(tName)s (%(idName)s) '
                       '%(action)s' %
                       {'tName': tName,
                        'colName': self.dbName,
                        'idName': idName,
-                       'action': action})
-        sql = ' '.join([sql, constraint])
-        return sql
+                       'action': action,
+                       'sTName': sTName})
+        return constraint
+
+    def mysqlCreateSQL(self):
+        return SOKeyCol.mysqlCreateSQL(self)
 
     def sybaseCreateSQL(self):
         sql = SOKeyCol.sybaseCreateSQL(self)
@@ -825,6 +826,10 @@ class SOForeignKey(SOKeyCol):
         sql = ' '.join([sql, reference])
         return sql
 
+    def sybaseCreateReferenceConstraint(self):
+        # @@: Code from above should be moved here
+        return None
+
     def mssqlCreateSQL(self):
         sql = SOKeyCol.mssqlCreateSQL(self)
         other = findClass(self.foreignKey)
@@ -836,6 +841,10 @@ class SOForeignKey(SOKeyCol):
         sql = ' '.join([sql, reference])
         return sql
 
+    def mssqlCreateReferenceConstraint(self):
+        # @@: Code from above should be moved here
+        return None
+
     def maxdbCreateSQL(self):
         other = findClass(self.foreignKey)
         fidName = self.dbName
@@ -847,6 +856,10 @@ class SOForeignKey(SOKeyCol):
         sql=sql + 'FOREIGN KEY (%s) REFERENCES %s(%s)'%(fidName,tName,idName)
         return sql
 
+    def maxdbCreateReferenceConstraint(self):
+        # @@: Code from above should be moved here
+        return None
+
 class ForeignKey(KeyCol):
 
     baseClass = SOForeignKey
@@ -1169,6 +1182,47 @@ class TimeCol(Col):
     baseClass = SOTimeCol
 
 
+try:
+    from decimal import Decimal
+except ImportError:
+    Decimal = float
+
+class DecimalValidator(validators.Validator):
+    def to_python(self, value, state):
+        if value is None:
+            return None
+        if isinstance(value, (int, long, Decimal, sqlbuilder.SQLExpression)):
+            return value
+        if isinstance(value, float):
+            value = str(value)
+        connection = state.soObject._connection
+        if hasattr(connection, "decimalSeparator"):
+            value = value.replace(connection.decimalSeparator, ".")
+        try:
+            return Decimal(value)
+        except:
+            raise validators.Invalid("expected a Decimal in the DecimalCol '%s', got %s %r instead" % \
+                (self.name, type(value), value), value, state)
+
+    def from_python(self, value, state):
+        if value is None:
+            return None
+        if isinstance(value, float):
+            value = str(value)
+        if isinstance(value, (str, unicode)):
+            connection = state.soObject._connection
+            if hasattr(connection, "decimalSeparator"):
+                value = value.replace(connection.decimalSeparator, ".")
+            try:
+                return Decimal(value)
+            except:
+                raise validators.Invalid("can not parse Decimal value '%s' in the DecimalCol from '%s'" %
+                    (value, getattr(state, 'soObject', '(unknown)')), value, state)
+        if not isinstance(value, (int, long, Decimal, sqlbuilder.SQLExpression)):
+            raise validators.Invalid("expected a decimal in the DecimalCol '%s', got %s %r instead" % \
+                (self.name, type(value), value), value, state)
+        return value
+
 class SODecimalCol(SOCol):
 
     def __init__(self, **kw):
@@ -1183,6 +1237,10 @@ class SODecimalCol(SOCol):
     def _sqlType(self):
         return 'DECIMAL(%i, %i)' % (self.size, self.precision)
 
+    def createValidators(self):
+        return [DecimalValidator()] + \
+            super(SODecimalCol, self).createValidators()
+
 class DecimalCol(Col):
     baseClass = SODecimalCol
 
