@@ -11,7 +11,7 @@ class FirebirdConnection(DBAPI):
     schemes = [dbName]
 
     def __init__(self, host, db, user='sysdba',
-                 passwd='masterkey', autoCommit=1,
+                 password='masterkey', autoCommit=1,
                  dialect=None, role=None, charset=None, **kw):
         global kinterbasdb
         if kinterbasdb is None:
@@ -23,7 +23,7 @@ class FirebirdConnection(DBAPI):
         self.host = host
         self.db = db
         self.user = user
-        self.passwd = passwd
+        self.password = password
         if dialect:
             self.dialect = int(dialect)
         else:
@@ -43,7 +43,7 @@ class FirebirdConnection(DBAPI):
         if (path[0] == '/') and path[-3:].lower() not in ('fdb', 'gdb'):
             path = path[1:]
         path = path.replace('/', os.sep)
-        return cls(host, db=path, user=auth, passwd=password, **args)
+        return cls(host, db=path, user=auth, password=password, **args)
     connectionFromURI = classmethod(connectionFromURI)
 
     def _runWithConnection(self, meth, *args):
@@ -78,7 +78,7 @@ class FirebirdConnection(DBAPI):
             host=self.host,
             database=self.db,
             user=self.user,
-            password=self.passwd,
+            password=self.password,
             role=self.role,
             charset=self.charset,
             **extra
@@ -127,12 +127,17 @@ class FirebirdConnection(DBAPI):
         self.query('CREATE TABLE %s (\n%s\n)' % \
                    (soClass.sqlmeta.table, self.createColumns(soClass)))
         self.query("CREATE GENERATOR GEN_%s" % soClass.sqlmeta.table)
+        return []
+
+    def createReferenceConstraint(self, soClass, col):
+        return None
 
     def createColumn(self, soClass, col):
         return col.firebirdCreateSQL()
 
     def createIDColumn(self, soClass):
-        return '%s INT NOT NULL PRIMARY KEY' % soClass.sqlmeta.idName
+        key_type = {int: "INT", str: "TEXT"}[soClass.sqlmeta.idType]
+        return '%s %s NOT NULL PRIMARY KEY' % (soClass.sqlmeta.idName, key_type)
 
     def createIndexSQL(self, soClass, index):
         return index.firebirdCreateIndexSQL(soClass)
@@ -169,33 +174,30 @@ class FirebirdConnection(DBAPI):
         """
 
         fieldqry = """\
-        SELECT rf.RDB$FIELD_NAME as field,
-               t.RDB$TYPE_NAME as t,
-               f.RDB$FIELD_LENGTH as flength,
-               f.RDB$FIELD_SCALE as fscale,
-               rf.RDB$NULL_FLAG as nullAllowed,
-               coalesce(rf.RDB$DEFAULT_SOURCE, f.rdb$default_source) as thedefault,
-               f.RDB$FIELD_SUB_TYPE as blobtype
-        FROM RDB$RELATION_FIELDS rf
-        INNER JOIN RDB$FIELDS f ON rf.RDB$FIELD_SOURCE = f.RDB$FIELD_NAME
-        INNER JOIN RDB$TYPES t ON f.RDB$FIELD_TYPE = t.RDB$TYPE
-        WHERE rf.RDB$RELATION_NAME = '%s'
-        AND t.RDB$FIELD_NAME = 'RDB$FIELD_TYPE'"""
+        SELECT RDB$RELATION_FIELDS.RDB$FIELD_NAME as field,
+               RDB$TYPES.RDB$TYPE_NAME as t,
+               RDB$FIELDS.RDB$FIELD_LENGTH as flength,
+               RDB$FIELDS.RDB$FIELD_SCALE as fscale,
+               RDB$RELATION_FIELDS.RDB$NULL_FLAG as nullAllowed,
+               RDB$RELATION_FIELDS.RDB$DEFAULT_VALUE as thedefault,
+               RDB$FIELDS.RDB$FIELD_SUB_TYPE as blobtype
+        FROM RDB$RELATION_FIELDS
+        INNER JOIN RDB$FIELDS ON
+            (RDB$RELATION_FIELDS.RDB$FIELD_SOURCE = RDB$FIELDS.RDB$FIELD_NAME)
+        INNER JOIN RDB$TYPES ON (RDB$FIELDS.RDB$FIELD_TYPE =
+                                 RDB$TYPES.RDB$TYPE)
+        WHERE
+            (RDB$RELATION_FIELDS.RDB$RELATION_NAME = '%s')
+            AND (RDB$TYPES.RDB$FIELD_NAME = 'RDB$FIELD_TYPE')"""
 
         colData = self.queryAll(fieldqry % tableName.upper())
         results = []
         for field, t, flength, fscale, nullAllowed, thedefault, blobType in colData:
-            field = field.strip().lower()
-            if thedefault:
-                thedefault = thedefault.split(' ')[1]
-                if thedefault.startswith("'") and thedefault.endswith("'"):
-                    thedefault = thedefault[1:-1]
-            idName = str(soClass.sqlmeta.idName or 'id').upper()
-            if field.upper() == idName:
+            if field == 'id':
                 continue
             colClass, kw = self.guessClass(t, flength, fscale)
             kw['name'] = soClass.sqlmeta.style.dbColumnToPythonAttr(field).strip()
-            kw['dbName'] = field
+            kw['dbName'] = field.strip()
             kw['notNone'] = not nullAllowed
             kw['default'] = thedefault
             results.append(colClass(**kw))
