@@ -6,13 +6,11 @@ import os
 import sys
 import textwrap
 import warnings
-try:
-    from paste import pyconfig
-    from paste import CONFIG
-except ImportError, e:
-    pyconfig = None
-    CONFIG = {}
 import time
+try:
+    from paste.deploy import appconfig
+except ImportError:
+    appconfig = None
 
 import sqlobject
 from sqlobject import col
@@ -396,13 +394,15 @@ class Command(object):
     def config(self):
         if not getattr(self.options, 'config_file', None):
             return None
-        if pyconfig and self.options.config_fn.endswith('.conf'):
-            config = pyconfig.Config(with_default=True)
-            config.load(self.options.config_file)
-            CONFIG.push_process_config(config)
-            return config
+        config_file = self.options.config_file
+        if appconfig:
+            if (not config_file.startswith('egg:')
+                and not config_file.startswith('config:')):
+                config_file = 'config:' + config_file
+            return appconfig(config_file,
+                             relative_to=os.getcwd())
         else:
-            return self.ini_config(self.options.config_file)
+            return self.ini_config(config_file)
 
     def ini_config(self, conf_fn):
         conf_section = 'main'
@@ -502,7 +502,7 @@ class Command(object):
         if not dist.has_metadata('sqlobject.txt'):
             if warn_no_sqlobject:
                 print 'No sqlobject.txt in %s egg info' % egg_spec
-            return {}
+            return None, {}
         result = {}
         for line in dist.get_metadata_lines('sqlobject.txt'):
             line = line.strip()
@@ -511,7 +511,7 @@ class Command(object):
             name, value = line.split('=', 1)
             name = name.strip().lower()
             if name in result:
-                print 'Warning: %s appears more than one in sqlobject.txt' % name
+                print 'Warning: %s appears more than once in sqlobject.txt' % name
             result[name.strip().lower()] = value.strip()
         return dist, result
 
@@ -577,11 +577,20 @@ class CommandSQL(Command):
 
     def command(self):
         classes = self.classes()
+        allConstraints = []
         for cls in classes:
             if self.options.verbose >= 1:
                 print '-- %s from %s' % (
                     cls.__name__, cls.__module__)
-            print cls.createTableSQL().strip() + ';\n'
+            createSql, constraints = cls.createTableSQL()
+            print createSql.strip() + ';\n'
+            allConstraints.append(constraints)
+        for constraints in allConstraints:
+            if constraints:
+                for constraint in constraints:
+                    if constraint:
+                        print constraint.strip() + ';\n'
+        
 
 class CommandList(Command):
 
@@ -615,6 +624,7 @@ class CommandCreate(Command):
         created = 0
         existing = 0
         dbs_created = []
+        constraints = {}
         for soClass in self.classes(require_some=True):
             if (self.options.create_db
                 and soClass._connection not in dbs_created):
@@ -629,6 +639,8 @@ class CommandCreate(Command):
                 else:
                     print '(simulating; cannot create database)'
                 dbs_created.append(soClass._connection)
+            if soClass._connection not in constraints.keys():
+                constraints[soClass._connection] = []
             exists = soClass._connection.tableExists(soClass.sqlmeta.table)
             if v >= 1:
                 if exists:
@@ -637,18 +649,30 @@ class CommandCreate(Command):
                 else:
                     print 'Creating %s' % soClass.__name__
             if v >= 2:
-                print soClass.createTableSQL()
+                sql, extra = soClass.createTableSQL()
+                print sql
             if (not self.options.simulate
                 and not exists):
                 if self.options.interactive:
                     if self.ask('Create %s' % soClass.__name__):
                         created += 1
-                        soClass.createTable()
+                        tableConstraints = soClass.createTable()
+                        if tableConstraints:
+                            constraints[soClass._connection].append(tableConstraints)
                     else:
                         print 'Cancelled'
                 else:
                     created += 1
-                    soClass.createTable()
+                    tableConstraints = soClass.createTable()
+                    if tableConstraints:
+                        constraints[soClass._connection].append(tableConstraints)
+        for connection in constraints.keys():
+            if v >= 2:
+                print 'Creating constraints'
+            for constraintList in constraints[connection]:
+                for constraint in constraintList:
+                    if constraint:
+                        connection.query(constraint)
         if v >= 1:
             print '%i tables created (%i already exist)' % (
                 created, existing)
@@ -918,15 +942,7 @@ class CommandRecord(Command):
                               + '_' + dbName + '.sql')
             if sim:
                 continue
-            # @@: This is a little hacky, but sometimes I'm getting string
-            # results here, and sometimes tuples (apparently, if the code is 
-            # meant to work?), so we'll allow for both
-            result = cls.createTableSQL()
-            if isinstance(result, basestring):
-                create = result
-                constraints = []
-            else:
-                create, constraints = result
+            create, constraints = cls.createTableSQL()
             if constraints:
                 constraints = '\n-- Constraints:\n%s\n' % (
                     '\n'.join(constraints))
@@ -990,25 +1006,25 @@ class CommandRecord(Command):
                 f = open(os.path.join(output_dir, fn), 'w')
                 f.write(content)
                 f.close()
-        all_diffs = []
-        for cls in self.classes():
-            for conn in conns:
-                diffs = db_differences(cls, conn)
-                for diff in diffs:
-                    if len(conns) > 1:
-                        diff = '  (%s).%s: %s' % (
-                            conn.uri(), cls.sqlmeta.table, diff)
-                    else:
-                        diff = '  %s: %s' % (cls.sqlmeta.table, diff)
-                    all_diffs.append(diff)
-        if all_diffs:
-            print 'Database does not match schema:'
-            print '\n'.join(all_diffs)
-            if self.options.db_record:
-                print '(Not updating database version)'
-        elif self.options.db_record:
-            for conn in conns:
-                self.update_db(version, conn)
+        if self.options.db_record:
+            all_diffs = []
+            for cls in self.classes():
+                for conn in conns:
+                    diffs = db_differences(cls, conn)
+                    for diff in diffs:
+                        if len(conns) > 1:
+                            diff = '  (%s).%s: %s' % (
+                                conn.uri(), cls.sqlmeta.table, diff)
+                        else:
+                            diff = '  %s: %s' % (cls.sqlmeta.table, diff)
+                        all_diffs.append(diff)
+            if all_diffs:
+                print 'Database does not match schema:'
+                print '\n'.join(all_diffs)
+                for conn in conns:
+                    self.update_db(version, conn)
+        else:
+            all_diffs = []
         if self.options.open_editor:
             if not last_version_dir:
                 print ("Cannot edit upgrader because there is no "
@@ -1142,7 +1158,11 @@ class CommandUpgrade(CommandRecord):
         if self.options.upgrade_to:
             version_to = self.options.upgrade_to
         else:
-            version_to = os.path.basename(self.find_last_version())
+            fname = self.find_last_version()
+            if fname is None:
+                print "No version exists, use 'record' command to create one"
+                return
+            version_to = os.path.basename(fname)
         current = self.current_version()
         if v:
             print 'Current version: %s' % current
