@@ -1,16 +1,17 @@
 """
-Contributed by Edigram SAS,Paris  France
-Tel:01 44 77 94 00
+Contributed by Edigram SAS, Paris France Tel:01 44 77 94 00
 Ahmed MOHAMED ALI <ahmedmoali@yahoo.com> 27 April 2004
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
 
-connection creation sample
-__connection__ = DBConnection.maxdbConnection(
-    host=hostname, database=dbname,
-    user=user_name, password=user_password,autoCommit=1,debug=1)
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+connection creation sample::
+
+    __connection__ = DBConnection.maxdbConnection(
+        host=hostname, database=dbname,
+        user=user_name, password=user_password,autoCommit=1,debug=1)
 """
 from sqlobject.dbconnection import DBAPI
 from sqlobject import col
@@ -80,7 +81,7 @@ class MaxdbConnection(DBAPI):
     def connectionFromURI(cls, uri):
         auth, password, host, port, path, args = cls._parseURI(uri)
         path = path.replace('/', os.path.sep)
-        return cls(host, db=path, user=auth, passwd=password, **args)
+        return cls(host, db=path, user=auth, password=password, **args)
     connectionFromURI = classmethod(connectionFromURI)
 
     def _getConfigParams(self,sqlmode,auto):
@@ -167,12 +168,17 @@ class MaxdbConnection(DBAPI):
                    (soClass.sqlmeta.table, self.createColumns(soClass)))
         self.query("CREATE SEQUENCE %s"
                    % self.createSequenceName(soClass.sqlmeta.table))
+        return []
+
+    def createReferenceConstraint(self, soClass, col):
+        return col.maxdbCreateReferenceConstraint()
 
     def createColumn(self, soClass, col):
         return col.maxdbCreateSQL()
 
     def createIDColumn(self, soClass):
-        return '%s INT PRIMARY KEY' % soClass.sqlmeta.idName
+        key_type = {int: "INT", str: "TEXT"}[soClass.sqlmeta.idType]
+        return '%s %s PRIMARY KEY' % (soClass.sqlmeta.idName, key_type)
 
     def createIndexSQL(self, soClass, index):
         return index.maxdbCreateIndexSQL(soClass)
