@@ -1,25 +1,37 @@
-from sqlobject import col
 from sqlobject.dbconnection import DBAPI
-from sqlobject.main import deprecated
+from sqlobject.dberrors import *
+from sqlobject import col
 MySQLdb = None
 
+class ErrorMessage(str):
+    def __new__(cls, e):
+        obj = str.__new__(cls, e[1])
+        obj.code = int(e[0])
+        obj.module = e.__module__
+        obj.exception = e.__class__.__name__
+        return obj
+
 class MySQLConnection(DBAPI):
 
     supportTransactions = False
     dbName = 'mysql'
     schemes = [dbName]
 
-    def __init__(self, db, user, passwd='', host='localhost', port=None, **kw):
+    def __init__(self, db, user, password='', host='localhost', port=None, **kw):
         global MySQLdb
         if MySQLdb is None:
-            import MySQLdb
+            import MySQLdb, MySQLdb.constants.CR, MySQLdb.constants.ER
         self.module = MySQLdb
         self.host = host
         self.port = port
         self.db = db
         self.user = user
-        self.password = passwd
+        self.password = password
         self.kw = {}
+        if MySQLdb.version_info[:3] >= (1, 2, 1):
+            self.need_unicode = True
+        else:
+            self.need_unicode = False
         for key in ("unix_socket", "init_command",
                 "read_default_file", "read_default_group"):
             if key in kw:
@@ -33,32 +45,25 @@ class MySQLConnection(DBAPI):
         else:
             self.dbEncoding = None
         if "sqlobject_encoding" in kw:
-            del kw["sqlobject_encoding"]
-            deprecated("sqlobject_encoding is deprecated and no longer used.")
+            self.encoding = col.popKey(kw, "sqlobject_encoding")
+        else:
+            self.encoding = 'ascii'
         DBAPI.__init__(self, **kw)
 
     def connectionFromURI(cls, uri):
         user, password, host, port, path, args = cls._parseURI(uri)
-        return cls(db=path.strip('/'), user=user or '', passwd=password or '',
+        return cls(db=path.strip('/'), user=user or '', password=password or '',
                    host=host or 'localhost', port=port or 0, **args)
     connectionFromURI = classmethod(connectionFromURI)
 
     def makeConnection(self):
-        dbEncoding = self.dbEncoding
-        if dbEncoding:
-            from MySQLdb.connections import Connection
-            if not hasattr(Connection, 'set_character_set'):
-                # monkeypatch pre MySQLdb 1.2.1
-                def character_set_name(self):
-                    return dbEncoding + '_' + dbEncoding
-                Connection.character_set_name = character_set_name
         try:
             conn = self.module.connect(host=self.host, port=self.port,
                 db=self.db, user=self.user, passwd=self.password, **self.kw)
             if MySQLdb.version_info[:3] >= (1, 2, 2):
                 conn.ping(True) # Attempt to reconnect. This setting is persistent.
         except self.module.OperationalError, e:
-            raise self.module.OperationalError(
+            raise OperationalError(
                 "%s; used connection string: host=%s, port=%s, db=%s, user=%s, pwd=%s" % (
                 e, self.host, self.port, self.db, self.user, self.password)
             )
@@ -66,13 +71,6 @@ class MySQLConnection(DBAPI):
         if hasattr(conn, 'autocommit'):
             conn.autocommit(bool(self.autoCommit))
 
-        if dbEncoding:
-            if hasattr(conn, 'set_character_set'): # MySQLdb 1.2.1 and later
-                conn.set_character_set(dbEncoding)
-            else: # pre MySQLdb 1.2.1
-                # works along with monkeypatching code above
-                conn.query("SET NAMES %s" % dbEncoding)
-
         return conn
 
     def _setAutoCommit(self, conn, auto):
@@ -80,15 +78,57 @@ class MySQLConnection(DBAPI):
             conn.autocommit(auto)
 
     def _executeRetry(self, conn, cursor, query):
-        while 1:
+        # When a server connection is lost and a query is attempted, most of
+        # the time the query will raise a SERVER_LOST exception, then at the
+        # second attempt to execute it, the mysql lib will reconnect and
+        # succeed. However is a few cases, the first attempt raises the
+        # SERVER_GONE exception, the second attempt the SERVER_LOST exception
+        # and only the third succeeds. Thus the 3 in the loop count.
+        # If it doesn't reconnect even after 3 attempts, while the database is
+        # up and running, it is because a 5.0.3 (or newer) server is used
+        # which no longer permits autoreconnects by default. In that case a
+        # reconnect flag must be set when making the connection to indicate
+        # that autoreconnecting is desired. In MySQLdb 1.2.2 or newer this is
+        # done by calling ping(True) on the connection.
+        for count in range(3):
             try:
-                return cursor.execute(query)
+                if self.need_unicode:
+                    # For MysqlDB 1.2.1 and later, we go
+                    # encoding->unicode->charset (in the mysql db)
+                    myquery = unicode(query, self.encoding)
+                    return cursor.execute(myquery)
+                else:
+                    return cursor.execute(query)
             except MySQLdb.OperationalError, e:
-                if e.args[0] == 2013: # SERVER_LOST error
+                if e.args[0] in (MySQLdb.constants.CR.SERVER_GONE_ERROR, MySQLdb.constants.CR.SERVER_LOST):
+                    if count == 2:
+                        raise OperationalError(ErrorMessage(e))
                     if self.debug:
                         self.printDebug(conn, str(e), 'ERROR')
                 else:
-                    raise
+                    raise OperationalError(ErrorMessage(e))
+            except MySQLdb.IntegrityError, e:
+                msg = ErrorMessage(e)
+                if e.args[0] == MySQLdb.constants.ER.DUP_ENTRY:
+                    raise DuplicateEntryError(msg)
+                else:
+                    raise IntegrityError(msg)
+            except MySQLdb.InternalError, e:
+                raise InternalError(ErrorMessage(e))
+            except MySQLdb.ProgrammingError, e:
+                raise ProgrammingError(ErrorMessage(e))
+            except MySQLdb.DataError, e:
+                raise DataError(ErrorMessage(e))
+            except MySQLdb.NotSupportedError, e:
+                raise NotSupportedError(ErrorMessage(e))
+            except MySQLdb.DatabaseError, e:
+                raise DatabaseError(ErrorMessage(e))
+            except MySQLdb.InterfaceError, e:
+                raise InterfaceError(ErrorMessage(e))
+            except MySQLdb.Warning, e:
+                raise Warning(ErrorMessage(e))
+            except MySQLdb.Error, e:
+                raise Error(ErrorMessage(e))
 
     def _queryInsertID(self, conn, soInstance, id, names, values):
         table = soInstance.sqlmeta.table
@@ -117,6 +157,9 @@ class MySQLConnection(DBAPI):
             return "%s LIMIT %i, -1" % (query, start)
         return "%s LIMIT %i, %i" % (query, start, end-start)
 
+    def createReferenceConstraint(self, soClass, col):
+        return col.mysqlCreateReferenceConstraint()
+
     def createColumn(self, soClass, col):
         return col.mysqlCreateSQL()
 
@@ -124,6 +167,8 @@ class MySQLConnection(DBAPI):
         return index.mysqlCreateIndexSQL(soClass)
 
     def createIDColumn(self, soClass):
+        if soClass.sqlmeta.idType == str:
+            return '%s TEXT PRIMARY KEY' % soClass.sqlmeta.idName
         return '%s INT PRIMARY KEY AUTO_INCREMENT' % soClass.sqlmeta.idName
 
     def joinSQLType(self, join):
@@ -136,8 +181,8 @@ class MySQLConnection(DBAPI):
             # which is not always True (for an embedded application, e.g.)
             self.query('DESCRIBE %s' % (tableName))
             return True
-        except MySQLdb.ProgrammingError, e:
-            if e.args[0] == 1146: # ER_NO_SUCH_TABLE
+        except ProgrammingError, e:
+            if e[0].code == 1146: # ER_NO_SUCH_TABLE
                 return False
             raise
 
@@ -161,20 +206,12 @@ class MySQLConnection(DBAPI):
             colClass, kw = self.guessClass(t)
             if self.kw.get('use_unicode') and colClass is col.StringCol:
                 colClass = col.UnicodeCol
+                if self.dbEncoding: kw['dbEncoding'] = self.dbEncoding
             kw['name'] = soClass.sqlmeta.style.dbColumnToPythonAttr(field)
             kw['dbName'] = field
-
-            # Since MySQL 5.0, 'NO' is returned in the NULL column (SQLObject expected '')
-            kw['notNone'] = (nullAllowed.upper() != 'YES' and True or False)
-
+            kw['notNone'] = not nullAllowed
             if default and t.startswith('int'):
                 kw['default'] = int(default)
-            elif default and t.startswith('float'):
-                kw['default'] = float(default)
-            elif default == 'CURRENT_TIMESTAMP' and t == 'timestamp':
-                kw['default'] = None
-            elif default and colClass is col.BoolCol:
-                kw['default'] = int(default) and True or False
             else:
                 kw['default'] = default
             # @@ skip key...
@@ -185,22 +222,12 @@ class MySQLConnection(DBAPI):
     def guessClass(self, t):
         if t.startswith('int'):
             return col.IntCol, {}
-        elif t.startswith('enum'):
-            values = []
-            for i in t[5:-1].split(','): # take the enum() off and split
-                values.append(i[1:-1]) # remove the surrounding \'
-            return col.EnumCol, {'enumValues': values}
-        elif t.startswith('double'):
-            return col.FloatCol, {}
         elif t.startswith('varchar'):
-            colType = col.StringCol
-            if self.kw.get('use_unicode', False):
-                colType = col.UnicodeCol
             if t.endswith('binary'):
-                return colType, {'length': int(t[8:-8]),
+                return col.StringCol, {'length': int(t[8:-8]),
                                        'char_binary': True}
             else:
-                return colType, {'length': int(t[8:-1])}
+                return col.StringCol, {'length': int(t[8:-1])}
         elif t.startswith('char'):
             if t.endswith('binary'):
                 return col.StringCol, {'length': int(t[5:-8]),
@@ -211,8 +238,6 @@ class MySQLConnection(DBAPI):
                                        'varchar': False}
         elif t.startswith('datetime'):
             return col.DateTimeCol, {}
-        elif t.startswith('time'):
-            return col.TimeCol, {}
         elif t.startswith('bool'):
             return col.BoolCol, {}
         elif t.startswith('tinyblob'):
