@@ -72,11 +72,14 @@ True, False = (1==1), (0==1)
 import re, fnmatch
 import operator
 import threading
+import types
+
 from converters import sqlrepr, registerConverter, TRUE, FALSE
 
 safeSQLRE = re.compile(r'^[a-zA-Z_][a-zA-Z0-9_\.]*$')
 def sqlIdentifier(obj):
-    return type(obj) is type("") and not not safeSQLRE.search(obj.strip())
+    # some db drivers return unicode column names 
+    return isinstance(obj, types.StringTypes) and bool(safeSQLRE.search(obj.strip()))
 
 
 def execute(expr, executor):
@@ -627,6 +630,8 @@ class DESC(SQLExpression):
         return '%s DESC' % sqlrepr(self.expr, db)
 
 def AND(*ops):
+    if not ops:
+        return None
     op1 = ops[0]
     ops = ops[1:]
     if ops:
@@ -635,6 +640,8 @@ def AND(*ops):
         return op1
 
 def OR(*ops):
+    if not ops:
+        return None
     op1 = ops[0]
     ops = ops[1:]
     if ops:
