@@ -1,12 +1,20 @@
-import base64
 from sqlobject.dbconnection import DBAPI
 from sqlobject.col import popKey
+from sqlobject.dberrors import *
 import thread
 
 sqlite = None
 using_sqlite2 = False
 sqlite2_Binary = None
 
+class ErrorMessage(str):
+    def __new__(cls, e):
+        obj = str.__new__(cls, e[0])
+        obj.code = None
+        obj.module = e.__module__
+        obj.exception = e.__class__.__name__
+        return obj
+
 class SQLiteConnection(DBAPI):
 
     supportTransactions = True
@@ -18,13 +26,15 @@ class SQLiteConnection(DBAPI):
         global using_sqlite2
         if sqlite is None:
             try:
-                from pysqlite2 import dbapi2 as sqlite
-                sqlite.encode = base64.encodestring
-                sqlite.decode = base64.decodestring
+                import sqlite3 as sqlite
                 using_sqlite2 = True
             except ImportError:
-                import sqlite
-                using_sqlite2 = False
+                try:
+                    from pysqlite2 import dbapi2 as sqlite
+                    using_sqlite2 = True
+                except ImportError:
+                    import sqlite
+                    using_sqlite2 = False
         self.module = sqlite
         self.filename = filename  # full path to sqlite-db-file
         self._memory = filename == ':memory:'
@@ -44,10 +54,24 @@ class SQLiteConnection(DBAPI):
             for col_type in "text", "char", "varchar", "date", "time", "datetime", "timestamp":
                 sqlite.register_converter(col_type, stop_pysqlite2_converting_strings)
                 sqlite.register_converter(col_type.upper(), stop_pysqlite2_converting_strings)
+            try:
+                from sqlite import encode, decode
+            except ImportError:
+                import base64
+                sqlite.encode = base64.encodestring
+                sqlite.decode = base64.decodestring
+            else:
+                sqlite.encode = encode
+                sqlite.decode = decode
             global sqlite2_Binary
             if sqlite2_Binary is None:
                 sqlite2_Binary = sqlite.Binary
                 sqlite.Binary = lambda s: sqlite2_Binary(sqlite.encode(s))
+            if 'factory' in kw:
+                factory = popKey(kw, 'factory')
+                if isinstance(factory, str):
+                    factory = globals()[factory]
+                opts['factory'] = factory(sqlite)
         else:
             opts['autocommit'] = bool(autoCommit)
             if 'encoding' in kw:
@@ -148,6 +172,36 @@ class SQLiteConnection(DBAPI):
             return self._memoryConn
         return sqlite.connect(self.filename, **self._connOptions)
 
+    def _executeRetry(self, conn, cursor, query):
+        if self.debug:
+            self.printDebug(conn, query, 'QueryR')
+        try:
+            return cursor.execute(query)
+        except self.module.OperationalError, e:
+            raise OperationalError(ErrorMessage(e))
+        except self.module.IntegrityError, e:
+            msg = ErrorMessage(e)
+            if msg.startswith('column') and msg.endswith('not unique'):
+                raise DuplicateEntryError(msg)
+            else:
+                raise IntegrityError(msg)
+        except self.module.InternalError, e:
+            raise InternalError(ErrorMessage(e))
+        except self.module.ProgrammingError, e:
+            raise ProgrammingError(ErrorMessage(e))
+        except self.module.DataError, e:
+            raise DataError(ErrorMessage(e))
+        except self.module.NotSupportedError, e:
+            raise NotSupportedError(ErrorMessage(e))
+        except self.module.DatabaseError, e:
+            raise DatabaseError(ErrorMessage(e))
+        except self.module.InterfaceError, e:
+            raise InterfaceError(ErrorMessage(e))
+        except self.module.Warning, e:
+            raise Warning(ErrorMessage(e))
+        except self.module.Error, e:
+            raise Error(ErrorMessage(e))
+
     def _queryInsertID(self, conn, soInstance, id, names, values):
         table = soInstance.sqlmeta.table
         idName = soInstance.sqlmeta.idName
@@ -158,7 +212,7 @@ class SQLiteConnection(DBAPI):
         q = self._insertSQL(table, names, values)
         if self.debug:
             self.printDebug(conn, q, 'QueryIns')
-        c.execute(q)
+        self._executeRetry(conn, c, q)
         # lastrowid is a DB-API extension from "PEP 0249":
         if id is None:
             id = int(c.lastrowid)
@@ -185,8 +239,12 @@ class SQLiteConnection(DBAPI):
     def createColumn(self, soClass, col):
         return col.sqliteCreateSQL()
 
+    def createReferenceConstraint(self, soClass, col):
+        return None
+
     def createIDColumn(self, soClass):
-        return '%s INTEGER PRIMARY KEY' % soClass.sqlmeta.idName
+        key_type = {int: "INTEGER", str: "TEXT"}[soClass.sqlmeta.idType]
+        return '%s %s PRIMARY KEY' % (soClass.sqlmeta.idName, key_type)
 
     def joinSQLType(self, join):
         return 'INT NOT NULL'
