@@ -1,5 +1,6 @@
 import sqlbuilder
 import dbconnection
+import main
 
 StringType = type('')
 
@@ -20,26 +21,24 @@ class SelectResults(object):
         self.clauseTables = clauseTables
         self.tables = tablesDict.keys()
         self.ops = ops
-        if ops.get('orderBy', sqlbuilder.NoDefault) is sqlbuilder.NoDefault:
-            ops['orderBy'] = sourceClass.sqlmeta.defaultOrder
-        orderBy = ops['orderBy']
+        if self.ops.get('orderBy', sqlbuilder.NoDefault) is sqlbuilder.NoDefault:
+            self.ops['orderBy'] = sourceClass.sqlmeta.defaultOrder
+        orderBy = self.ops['orderBy']
         if isinstance(orderBy, list) or isinstance(orderBy, tuple):
             orderBy = map(self._mungeOrderBy, orderBy)
         else:
             orderBy = self._mungeOrderBy(orderBy)
-        ops['dbOrderBy'] = orderBy
+        self.ops['dbOrderBy'] = orderBy
         if ops.has_key('connection') and ops['connection'] is None:
             del ops['connection']
-        if ops.get('limit', None):
-            assert not ops.get('start', None) and not ops.get('end', None), \
-               "'limit' cannot be used with 'start' or 'end'"
-            ops["start"] = 0
-            ops["end"] = ops["limit"]
-            del ops["limit"]
 
     def __repr__(self):
         return "<%s at %x>" % (self.__class__.__name__, id(self))
 
+    def __nonzero__(self):
+         raise NotImplementedError(
+             "To test if a SelectResult will produce any items, use list(result) or result.count()")
+
     def _getConnection(self):
         return self.ops.get('connection') or self.sourceClass._connection
 
@@ -179,11 +178,9 @@ class SelectResults(object):
 
     def count(self):
         """ Counting elements of current select results """
-        assert not self.ops.get('start') and not self.ops.get('end'), \
-            "start/end/limit have no meaning with 'count'"
         assert not (self.ops.get('distinct') and (self.ops.get('start')
                                                   or self.ops.get('end'))), \
-            "distinct-counting of sliced objects is not supported"
+               "distinct-counting of sliced objects is not supported"
         if self.ops.get('distinct'):
             # Column must be specified, so we are using unique ID column.
             # COUNT(DISTINCT column) is supported by MySQL and PostgreSQL,
@@ -235,4 +232,29 @@ class SelectResults(object):
     def max(self, attribute):
         return self.accumulateOne("MAX", attribute)
 
+    def getOne(self, default=sqlbuilder.NoDefault):
+        """
+        If a query is expected to only return a single value,
+        using ``.getOne()`` will return just that value.
+
+        If not results are found, ``SQLObjectNotFound`` will be
+        raised, unless you pass in a default value (like
+        ``.getOne(None)``).
+
+        If more than one result is returned,
+        ``SQLObjectIntegrityError`` will be raised.
+        """
+        results = list(self)
+        if not results:
+            if default is sqlbuilder.NoDefault:
+                raise main.SQLObjectNotFound(
+                    "No results matched the query for %s"
+                    % self.sourceClass.__name__)
+            return default
+        if len(results) > 1:
+            raise main.SQLObjectIntegrityError(
+                "More than one result returned from query: %s"
+                % results)
+        return results[0]
+
 __all__ = ['SelectResults']
