@@ -1,7 +1,9 @@
 from __future__ import generators # for enumerate
 from sqlobject import *
-from sqlobject.sqlbuilder import func
-from dbtest import *
+from sqlobject.main import SQLObjectIntegrityError
+from sqlobject.tests.dbtest import *
+from sqlobject.tests.dbtest import setSQLiteConnectionFactory
+from py.test import raises
 
 try:
     enumerate
@@ -72,18 +74,17 @@ def test_04_indexed_ended_by_exception():
         pass
     assert count == len(names)
 
-def test_05_select_limit():
-    setupIter()
-    assert len(list(IterTest.select(limit=2))) == 2
-    raises(AssertionError, IterTest.select(limit=2).distinct)
-    raises(AssertionError, IterTest.select(limit=2).clone, start=1)
-
-def test_06_like():
-    setupIter()
-    assert len(list(IterTest.select(IterTest.q.name.startswith('a')))) == 1
-    assert len(list(IterTest.select(IterTest.q.name.endswith('a')))) == 1
-    assert len(list(IterTest.select(IterTest.q.name.contains('a')))) == 1
-    assert len(list(IterTest.select(IterTest.q.name.contains(func.lower('A'))))) == 1
+def test_select_getOne():
+    setupClass(IterTest)
+    a = IterTest(name='a')
+    b = IterTest(name='b')
+    assert IterTest.selectBy(name='a').getOne() == a
+    assert IterTest.select(IterTest.q.name=='b').getOne() == b
+    assert IterTest.selectBy(name='c').getOne(None) is None
+    raises(SQLObjectNotFound, 'IterTest.selectBy(name="c").getOne()')
+    b2 = IterTest(name='b')
+    raises(SQLObjectIntegrityError, 'IterTest.selectBy(name="b").getOne()')
+    raises(SQLObjectIntegrityError, 'IterTest.selectBy(name="b").getOne(None)')
 
 def test_selectBy():
     setupClass(IterTest)
@@ -133,7 +134,7 @@ class TestSelect:
     def test_2(self):
         self.accumulateEqual(len,Counter2.select('all'), 100)
 
-def test_sqlbuilder_LIKE():
+def test_select_LIKE():
     setupClass(IterTest)
     IterTest(name='sqlobject')
     IterTest(name='sqlbuilder')
@@ -142,12 +143,12 @@ def test_sqlbuilder_LIKE():
     assert IterTest.select(LIKE(IterTest.q.name, "sqlb%")).count() == 1
     assert IterTest.select(LIKE(IterTest.q.name, "sqlx%")).count() == 0
 
-def test_sqlbuilder_RLIKE():
+def test_select_RLIKE():
     setupClass(IterTest)
 
     if IterTest._connection.dbName == "sqlite":
-        from sqlobject.sqlite.sqliteconnection import using_sqlite2
-        if not using_sqlite2:
+        from sqlobject.sqlite import sqliteconnection
+        if not sqliteconnection.using_sqlite2:
             return
 
         # Implement regexp() function for SQLite; only works with PySQLite2
@@ -155,12 +156,14 @@ def test_sqlbuilder_RLIKE():
         def regexp(regexp, test):
             return bool(re.search(regexp, test))
 
-        _get_connection = IterTest._connection.getConnection
-        def new_get_connection(*args, **kw):
-            _connection = _get_connection(*args, **kw)
-            _connection.create_function("regexp", 2, regexp)
-            return _connection
-        IterTest._connection.getConnection = new_get_connection
+        def SQLiteConnectionFactory(sqlite):
+            class MyConnection(sqlite.Connection):
+                def __init__(self, *args, **kwargs):
+                    super(MyConnection, self).__init__(*args, **kwargs)
+                    self.create_function("regexp", 2, regexp)
+            return MyConnection
+
+        setSQLiteConnectionFactory(IterTest, SQLiteConnectionFactory)
 
     IterTest(name='sqlobject')
     IterTest(name='sqlbuilder')
@@ -168,3 +171,8 @@ def test_sqlbuilder_RLIKE():
     assert IterTest.select(RLIKE(IterTest.q.name, "^sqlb.*$")).count() == 1
     assert IterTest.select(RLIKE(IterTest.q.name, "^sqlb.*$")).count() == 1
     assert IterTest.select(RLIKE(IterTest.q.name, "^sqlx.*$")).count() == 0
+
+def test_bool_SelectResults():
+    setupClass(IterTest)
+
+    raises(NotImplementedError, bool, IterTest.select())
